function evol_nb_trans(id_trans,time_flag)
% EVOL_NB_TRANS(ID_TRANS,TIME_FLAG)
%   Extract the number of transitions with id number ID_TRANS from ReSCAL logs
%   and plot the evolution.
%   If TIME_FLAG is set, read time data in t0 unit from TIME.log file.

if ~exist('time_flag','var')
  time_flag=0;
end

%% read header of TRANS.log
fprintf('reading TRANS.log\n');
fid=fopen('TRANS.log');
if (fid==-1)
  fprintf('file not found\n');
  return
end
textscan(fid,'# TRANSITIONS');
C=textscan(fid,'NB_TRANS_DB = %d');
nb_trans=C{1};
for i=1:nb_trans
  CTrans=textscan(fid,'TR%[^\n]s');
end

%% skip line of text
C=textscan(fid,'%[^\n]s');
%celldisp(C)

C=textscan(fid,'NB_TRANS_CEL = %d');
if size(C{1})>0
  nb_tcel=C{1};
else
  nb_tcel=0;
end

for i=1:nb_tcel
  CTransCel=textscan(fid,'TR1%[^\n]s');
end

%% skip line of text
C=textscan(fid,'%[^\n]s');
%celldisp(C)

C=textscan(fid,'NB_LINKS = %d');
if size(C{1})>0
  nb_link=C{1};
else
  nb_link=0;
end
for i=1:nb_link
  CLinks=textscan(fid,'LT%[^\n]s');
end

C=textscan(fid,'%[^\n]s');
%celldisp(C)

%% read column headers
id=-1;
col_trans=-1;
nc=0;
%while (id~=id_trans) & (id~=nb_trans-1)
while (id~=nb_trans-1)
  C=textscan(fid,'TR( %d)');
  %celldisp(C)
  n=size(C{1},1);
  if n>0
    if (id < id_trans) & (id+n >= id_trans)
      col_trans=nc+(id_trans-id);
      fprintf('data for transition %d are located in column %d (+1)\n', id_trans, col_trans);
    end
    id=C{1}(n);
    nc=nc+n;
  end
  %C=textscan(fid,'%[BLK]s');
  %C=textscan(fid,'%[ ]BLK');
  C=textscan(fid,'B%[LK]');
  %keyboard
  %celldisp(C)
  if size(C{1},1)>0
    nc=nc+1;
  end
  %pause(1)
end

if nb_tcel>0
  id=-1;
  while (id~=nb_tcel-1)
    C=textscan(fid,'TR1( %d)');
    %celldisp(C);
    n=size(C{1},1);
    if n>0
      id=C{1}(n);
      nc=nc+n;
    end
  end
end

if nb_link>0
  id=-1;
  while (id~=nb_link-1)
    C=textscan(fid,'LT( %d)');
    %celldisp(C);
    n=size(C{1},1);
    if n>0
      id=C{1}(n);
      nc=nc+n;
    end
  end
end

fprintf('total number of columns: %d (+1)\n', nc);

%return

%% read trans data
C=textscan(fid,'%d','delimiter',':');
data0=cell2mat(C);
l=size(data0,1);
nb=l/(nc+1);
fprintf('number of records: %d\n', nb);
data=reshape(data0,nc+1,nb)';
transdata=data(:,col_trans+1);
fclose(fid);

%% read time data
if time_flag>0
  fprintf('reading TIME.log\n');
  fid=fopen('TIME.log');
  if (fid==-1)
    fprintf('file not found\n');
    return
  end
  C=textscan(fid,'%[^\n]s');
  %celldisp(C)
  C=textscan(fid,'%f','delimiter',':dhms');
  data0=cell2mat(C);
  l=size(data0,1);
  nb=nb+1;
  c=l/nb;
  data=reshape(data0,c,nb)';
  timedata=data(2:nb,4);
  fclose(fid);
end

%% plot data
figure
lw=0.5;
if time_flag==0
  plot(transdata,'linewidth',lw)
  xlabel('Computation time [min.]')
else
  plot(timedata,transdata,'linewidth',lw)
  xlabel('Time [t_{0}]')
end
title(sprintf('Number of occurences (per record) of transition %d', id_trans))
ylabel('Number of occurences')
grid on

avg=mean(transdata);
fprintf('average number of occurences per record: %f\n', avg);
if time_flag>0
  dt=max(timedata)-min(timedata);
  if dt>0
    avg_norm=avg*nb/dt;
    fprintf('mean rate of occurences: %f t_0^-1\n', avg_norm);
  end
end

return

