/* ReSCAL - Lattice gas
 *
 * Copyright (C) 2011
 *
 * Author: Olivier Rozier <rozier@ipgp.fr>
 *
 * This file is part of ReSCAL.
 *
 * ReSCAL is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifdef LGCA

/// save lgca raw binary data (HPP*.bin) with -dcsphpp option
//#define LGCA_SAVE_BINARY

/// forcing of the flow
//#define MVT_HFOR  //forcage horizontal du flux (tapis roulant)
//#define MVT_VFOR  //forcage partiel du flux sur le bord vertical gauche
#define MVT_GFOR  //forcage global du flux

/// solid-fluid interface
/// to set no-slip condition, comment the three lines below
//#define SOLID_SLIP   /// slip boundary condition on solid nodes
//#define SURFACE_SLIP   /// slip boundary condition on solid surface using normal vectors (obsolete)
#define CEIL_SLIP   /// slip boundary condition on the ceiling

//densite moyenne des particules en mouvement
#define DENSITE 1.5 //valeur <= 2.0

//masques pour les mouvements de fluides sur reseaux (modele multi-speed)

#define MVT_SOLID 1	//phase solide
#define MVT_OUT 2	//sortie
#define MVT_E 4		//est
#define MVT_O 8		//ouest
#define MVT_B 16	//bas
#define MVT_H 32	//haut
#define MVT_EB 64	//est-bas
#define MVT_EH 128	//est-haut
#define MVT_OB 256	//ouest-bas
#define MVT_OH 512	//ouest-haut

#define MVT_SLOW (MVT_E | MVT_O | MVT_B | MVT_H) //vitesses lentes
#define MVT_FAST (MVT_EB | MVT_EH | MVT_OB | MVT_OH) //vitesses rapides
#define MVT_ALLDIR (MVT_SLOW | MVT_FAST) //toute direction

typedef unsigned short MvtField;

#define SIZE_MVT_FIELD 1024 	//(1<<(8*sizeof(MvtField)))

//parametres pour le couplage et le moyennage

//#define NB_MVT_VER 1 //5  //nombre de noeuds verticalement par cellules
#define NB_MVT_EO 1 //5  //nombre de noeuds est-ouest par cellules
#ifndef STABILITY_ANALYSIS
#define DIST_MVT_NS 5 //5   //distance nord-sud entre 2 plans verticaux de collisions
#define DIST_MVT_H 5 //5   //distance entre 2 plans horizontaux de collisions
#else
#define DIST_MVT_NS 2    //distance nord-sud entre 2 plans verticaux de collisions (analyse de stabilite)
#define DIST_MVT_H 0    //distance entre 2 plans horizontaux de collisions
#endif

#define VSTEP 5    //pas pour le moyennage spatial (ATTENTION : VSTEP doit etre un multiple de NB_MVT_EO et NB_MVT_VER !!)
#define VSTEP_X VSTEP    //pas pour le moyennage spatial est-ouest
#define VSTEP_Y VSTEP    //pas pour le moyennage spatial vertical
#define VSTEP_TIME 50 //100  //pas pour le moyennage temporel
#define VMEAN_REF (VSTEP_TIME*VSTEP_TIME)/100 //seuil de vitesse moyenne (carre de la norme)

//constant enumerations

enum LGCA_BOUNDARY_CONDITIONS {LGCA_BC_PERIODIC, LGCA_BC_FLAT};

//macros

#define SetMask(mvt, mask) mvt |= mask
#define UnsetMask(mvt, mask) mvt &= (-1) ^ mask


/*
#define Calcul_cix(index, cind) \
{                           \
  int x,y,z;                  \
  Calcule_xyz(index, x, y, z); \
  cind = x*NB_MVT_EO + y*CLEO + ((z-LN)/DIST_MVT_NS)*Lsv.lxy; \
}
*/
#define Calcule_cix(index, cix) \
{ \
  int z = (int)index/HL; \
  int xy = index - z*HL; \
  int cz = (int)(z - LN)/DIST_MVT_NS; \
  cix = cz*Lsv.lxy + xy*NB_MVT_EO; \
}

#define Calcule_cxyz(index, cx, cy, cz) \
{                           \
  int x,y,z;                  \
  Calcule_xyz(index, x, y, z); \
  cx = x*NB_MVT_EO; \
  cy = y; \
  cz = (int)(z-LN)/DIST_MVT_NS; \
}

typedef struct{
  int orientation;
  int npmax;  /// max number of planes
  int lx, ly; /// plane dimensions
  int lxy;    /// size of planes
  int sz;   /// size of lgca space
  MvtField *mvt;  /// moving cells
  MvtField *mvt2; /// double buffering
  int nfc;  /// number of fluid cells
  float density; /// density of moving cells
  int pmin, pmax; /// indexes of first and last planes
  int np; /// number of planes
} LgcaSpace;

typedef struct{
  int orientation;
  int npmax;  /// max number of planes
  int lx, ly; /// velocity plane dimensions
  int lxy;    /// size of velocity planes
  int sz;    /// size of velocity space
  int *velx_time, *vely_time;  /// mean velocity spaces at different times
  int *velx, *vely;  /// mean velocity space (averaged over time)
  float *velx_interp, *vely_interp;  /// interpolated velocity space
  float maxvel; /// max of interpolated velocities
  float meanvel; /// mean of interpolated velocities
} VelSpace;

void params_lgca();
void init_lgca();
void init_mvt();
void out_of_space_mvt(int reset_mvt);
void collisions_mod_terre();
int mvt(int ii);
int mvt_solid(int ii);
int mvt_bas(int ii);
int mvt_haut(int ii);
int mvt_est(int ii);
int mvt_ouest(int ii);
//Callback_check check_mvt_bas;
//Callback_check check_mvt_haut;
//Callback_check check_mvt_ouest;
//Callback_check check_mvt_est;
int check_mvt_solid(int index, void *data);
int check_mvt_bas(int index, void *data);
int check_mvt_haut(int index, void *data);
int check_mvt_est(int index, void *data);
int check_mvt_ouest(int index, void *data);
int check_mvt_est_et_bas(int index, void *data);
int check_no_mvt_est_et_bas(int index, void *data);
void lgca_modify_cell(int type, int index);
void lgca_collisions();
void lgca_propagations();
void compute_vel(char flag_interp);
void dump_mvt(int cpt, int unit);
void dump_densite();
void dump_vel();
void dump_signature_mvt();

#endif //LGCA
